#!   /usr/bin/env   python

# Copyright Creotech SA, 2016
# Author: Marek Guminski <marek.guminski@creotech.pl>
# Licence: GPL v2 or later.
# Website: http://www.ohwr.org


import spec
import fmcmasterfip as fmc
import utilities as util
import ptsexcept
import time

DEL = 0.5
NRSAMPLES = 10

def test_sync_output( dut, box ):
    
    util.section_msg("Testing trigger in output mode")
    result = {}

#     enable output
    dut.fipcore.write_regname('ext_sync.oe', 1)
#     disable termination
    dut.fipcore.write_regname('ext_sync.term_en', 0)

#     set output value
    dut.fipcore.write_regname('ext_sync.output_value', 0)
#     set trigger to output direction
    dut.fipcore.write_regname('ext_sync.dir', 1)
    
    util.info_msg("Setting trigger to output")
    # connect trigger to channel 2 of ADC
    box.trigger_output_to_awg()
    
    util.info_msg("Connecting ADC to channel 1 of secondary transformer side ")
    # connect ADC to secondary transformer side (connected to trigger)
    dut.close_relay( 2 )
   
    # wait
    time.sleep(DEL)
    
    util.info_msg("")
    util.info_msg("Generating low output")
    result['Trigger output verified to go low'] = 1
    for i in xrange(NRSAMPLES):
        adc_samples = dut.adc_single_sample()
        
        
        if adc_samples[1] < util.test06_thd:
#             util.info_msg("Value read by ADC: CH1=%d, CH2=%d" %( adc_samples[0], adc_samples[1] ) )
            util.info_msg("Value read by ADC: %d" %( adc_samples[1] ) )
        else:
#             util.err_msg("Value read by ADC: CH1=%d, CH2=%d" %( adc_samples[0], adc_samples[1] ) )
            util.err_msg("Value read by ADC: %d" %( adc_samples[1] ) )
            result['Trigger output verified to go low'] = 0

        time.sleep(0.1)
    
    #######################

    util.info_msg("")
    util.info_msg("Generating high output")
    dut.fipcore.write_regname('ext_sync.output_value', 1)
    result['Trigger output verified to go high'] = 1
    time.sleep(DEL)
    
    # sample "1" output generated by sync buffer
    # do that for termination enabled and disabled
    # with termination enabled registered voltage should about 2x lower
    sampled = [0, 0]
    for t in xrange(2):
        # set termination (0 or 1)
        util.info_msg("")
        util.info_msg( "Termination : %d" % t)
        dut.fipcore.write_regname('ext_sync.term_en', t)
        
        for i in xrange(NRSAMPLES):
            adc_samples = dut.adc_single_sample()
            sampled[t]    += adc_samples[1];
            
            if adc_samples[1] >= util.test06_thd:
                util.info_msg("Value read by ADC: %d" % adc_samples[1]  )
                
            else:
                util.err_msg("Value read by ADC: %d" % adc_samples[1] )
                result['Trigger output verified to go high'] = 0
            time.sleep(0.1)
        
        time.sleep(DEL)
        
    util.info_msg("")
    util.info_msg("Mean value sampled without termination : %d" % (sampled[0]/NRSAMPLES) )
    util.info_msg("Mean value sampled with termination : %d" % (sampled[1]/NRSAMPLES) )
    
    output_ratio_term = float(sampled[0])/float(sampled[1])
    
    
    if output_ratio_term < util.test06_thdratio :
        util.err_msg("Ratio of output value with and without termination is %f. Should be over %f." % ( output_ratio_term, util.test06_thdratio ) )
        result['External sync termination'] = 0
    else :
        util.info_msg("Ratio of output value with and without termination is %f. Should be over %f." % ( output_ratio_term, util.test06_thdratio ) )
        result['External sync termination'] = 1
   
    time.sleep(DEL)
    

    #######################

    util.info_msg("\nDisabling bidir buffer output:")
    # disable output
    dut.fipcore.write_regname('ext_sync.oe', 0)
    time.sleep(DEL)
    
    for i in xrange(NRSAMPLES):
        adc_samples = dut.adc_single_sample()
        sampled[t]    += adc_samples[1];
            
        if adc_samples[1] <= util.test06_thd:
            util.info_msg("Value read by ADC: %d" % adc_samples[1]  )
            result['Read low value with bidir buffer OE disabled'] = 1
                
        else:
            util.err_msg("Output remains high when the bidir buffer OE is disabled. Value read by ADC: %d" % adc_samples[1])
            result['Read low value with bidir buffer OE disabled'] = 0
        time.sleep(0.1)
        
    time.sleep(DEL)
        
    util.info_msg("")
    util.info_msg("Output verified to go low when the bidir buffer OE is disabled.")


    #######################

    util.info_msg("\nEnabling bidir buffer output:")
    # disable output
    dut.fipcore.write_regname('ext_sync.oe', 1)
    
    for i in xrange(NRSAMPLES):
        adc_samples = dut.adc_single_sample()
        sampled[t]    += adc_samples[1];
            
        if adc_samples[1] >= util.test06_thd:
            util.info_msg("Value read by ADC: %d" % adc_samples[1]  )
            result['Read high value with bidir buffer OE enabled'] = 1
                
        else:
            util.err_msg("Output is high when the bidir buffer OE is enabled. Value read by ADC: %d" % adc_samples[1])
            result['Read high value with bidir buffer OE enabled'] = 0
        time.sleep(0.1)
        
    time.sleep(DEL)
        
    util.info_msg("")
    util.info_msg("Output verified to go high when the bidir buffer OE is enabled.")



    #######################

#     set output value low
    dut.fipcore.write_regname('ext_sync.output_value', 0)
    time.sleep(DEL)
    util.info_msg("")
    util.info_msg("Generating low output again")
    result['Trigger output verified to go low for the second time'] = 1
    for i in xrange(NRSAMPLES):
        adc_samples = dut.adc_single_sample()
        util.dbg_msg("Read ADC value: CH1=%d, CH2=%d low trigger output" %( adc_samples[0], adc_samples[1] ) )
        
        if adc_samples[1] < util.test06_thd:
            util.info_msg("Value read by ADC: %d" % adc_samples[1] )
        else:
            util.err_msg("Value read by ADC: %d" % adc_samples[1] )
            result['Trigger output verified to go low for the second time'] = 0
        time.sleep(0.1)   
        
    return result
    
    
###############################################################################

def test_sync_input( dut, box ):
    
    result = {}
    
    util.section_msg("Testing trigger in input mode")
    
    box.trigger_input_low()
    
    time.sleep(DEL)
    
    util.info_msg("Setting trigger to input")
#     set trigger to input direction
    dut.fipcore.write_regname('ext_sync.dir', 0)

#     enable tranceiver output
    dut.fipcore.write_regname('ext_sync.oe', 1)

#     enable termination (should bring input to gnd via resistors)
    dut.fipcore.write_regname('ext_sync.term_en', 1)
    
    time.sleep(DEL);
    
    if dut.fipcore.read_regname('ext_sync_raw_input') == 0 :
        util.info_msg("Value read from trigger input is 0")
        result['Read low value on trigger input with pulldown resistor enabled'] = 1
    else:
        util.err_msg("Value read from trigger input is 1")
        result['Read low value on trigger input with pulldown resistor enabled'] = 0
    
    time.sleep(DEL);
    
    #######################
    
    util.info_msg("")
    util.info_msg("Connecting input to reference voltage in calibration box")
    util.info_msg("Setting low input voltage")
    box.trigger_input_low()
    
#     set trigger to input direction
    dut.fipcore.write_regname('ext_sync.dir', 0)
#     enable output
    dut.fipcore.write_regname('ext_sync.oe', 1)
#     disable termination 
    dut.fipcore.write_regname('ext_sync.term_en', 0)
    
    time.sleep(DEL)
    if dut.fipcore.read_regname('ext_sync_raw_input') == 0: 
        result['Trigger input read low'] = 1
        util.info_msg("Read low input value") 
    else :
        util.err_msg("Read high input value")
        result['Trigger input read low'] = 0
        
    util.info_msg("")
    util.info_msg("Setting high input voltage")
    box.trigger_input_high()
    
    time.sleep(DEL)
    if dut.fipcore.read_regname('ext_sync_raw_input') == 1 :
        result['Trigger input read high'] = 1 
        util.info_msg("Read high input value")
    else :
        util.err_msg("Read low input value")
        result['Trigger input read high'] = 0 
        
    
    box.trigger_input_low()
    
    return result
    
def main (card=None, default_directory='.',suite=None, serial=""):
    
    testname= "Test06: Trigger Input/Output"
    util.header_msg( testname, [ "Bidirectional Buffer communication",
                                    "Termination switching",
                                    "Test pullup resistor switching",
                                    "Reading trigger input",
                                    "Generating trigger output" ] )
    
    ###############################################################################
    ############################ initialization ###################################
    ###############################################################################
    box = util.calibration_box_init()
    
    util.section_msg("Hardware initialization")
    abspath = util.find_prj_topdir(util.TOPDIRNAME);
    carrier = spec.spec( abspath );
        
   
    # FPGA bitstream and FMC presence are verified during initialization
    dut = fmc.fmcmasterfip(carrier, abspath, util.FIRMWARE_PATH)
    
    test_results = {}
    
    ###############################################################################
    ############################ actual test ######################################
    ###############################################################################
    
    
    
    
    test_results = util.merge_dictionaries( test_results, test_sync_input( dut, box ) )
    
    test_results = util.merge_dictionaries( test_results, test_sync_output( dut, box ) )
    
       
    ###############################################################################
    ########################## result processing ##################################
    ###############################################################################
    
    errors = util.summarise_test_results( testname, test_results)
        
if __name__ == '__main__' :
    main()
